@@ -3,7 +3,7 @@
 from boto.exception import JSONResponseError, BotoServerError
 
 from dynamic_dynamodb import calculators
-from dynamic_dynamodb.aws import dynamodb
+from dynamic_dynamodb.aws import dynamodb, sns
 from dynamic_dynamodb.core import circuit_breaker
 from dynamic_dynamodb.statistics import table as table_stats
 from dynamic_dynamodb.log_handler import LOGGER as logger
@@ -26,11 +26,15 @@ def ensure_provisioning(
     :param num_consec_write_checks: How many consecutive checks have we had
     :returns: (int, int) -- num_consec_read_checks, num_consec_write_checks
     """
+
     if get_global_option('circuit_breaker_url'):
         if circuit_breaker.is_open():
             logger.warning('Circuit breaker is OPEN!')
             return (0, 0)
 
+    # Handle throughput alarm checks
+    __ensure_provisioning_alarm(table_name, key_name)
+
     try:
         read_update_needed, updated_read_units, num_consec_read_checks = \
             __ensure_provisioning_reads(
@@ -135,12 +139,16 @@ def __ensure_provisioning_reads(table_name, key_name, num_consec_read_checks):
 
     update_needed = False
     try:
+        lookback_window_start = get_table_option(
+            key_name, 'lookback_window_start')
         current_read_units = dynamodb.get_provisioned_table_read_units(
             table_name)
         consumed_read_units_percent = \
-            table_stats.get_consumed_read_units_percent(table_name)
+            table_stats.get_consumed_read_units_percent(
+                table_name, lookback_window_start)
         throttled_read_count = \
-            table_stats.get_throttled_read_event_count(table_name)
+            table_stats.get_throttled_read_event_count(
+                table_name, lookback_window_start)
         reads_upper_threshold = \
             get_table_option(key_name, 'reads_upper_threshold')
         reads_lower_threshold = \
@@ -191,30 +199,69 @@ def __ensure_provisioning_reads(table_name, key_name, num_consec_read_checks):
             '{0} - Scaling down reads is not done when usage is at 0%'.format(
                 table_name))
 
+    # Increase needed due to high CU consumption
     elif consumed_read_units_percent >= reads_upper_threshold:
 
-        if increase_reads_unit == 'percent':
-            calculated_provisioning = calculators.increase_reads_in_percent(
-                current_read_units,
-                increase_reads_with,
-                get_table_option(key_name, 'max_provisioned_reads'),
-                table_name)
-        else:
-            calculated_provisioning = calculators.increase_reads_in_units(
-                current_read_units,
-                increase_reads_with,
-                get_table_option(key_name, 'max_provisioned_reads'),
-                table_name)
-
-        if current_read_units != calculated_provisioning:
-            logger.info(
-                '{0} - Resetting the number of consecutive '
-                'read checks. Reason: scale up event detected'.format(
+        # Exit if up scaling has been disabled
+        if not get_table_option(key_name, 'enable_reads_up_scaling'):
+            logger.debug(
+                '{0} - Up scaling event detected. No action taken as scaling '
+                'up reads has been disabled in the configuration'.format(
                     table_name))
-            num_consec_read_checks = 0
-            update_needed = True
-            updated_read_units = calculated_provisioning
+        else:
+            if increase_reads_unit == 'percent':
+                calculated_provisioning = calculators.increase_reads_in_percent(
+                    current_read_units,
+                    increase_reads_with,
+                    get_table_option(key_name, 'max_provisioned_reads'),
+                    table_name)
+            else:
+                calculated_provisioning = calculators.increase_reads_in_units(
+                    current_read_units,
+                    increase_reads_with,
+                    get_table_option(key_name, 'max_provisioned_reads'),
+                    table_name)
 
+            if current_read_units != calculated_provisioning:
+                logger.info(
+                    '{0} - Resetting the number of consecutive '
+                    'read checks. Reason: scale up event detected'.format(
+                        table_name))
+                num_consec_read_checks = 0
+                update_needed = True
+                updated_read_units = calculated_provisioning
+
+    # Decrease needed due to low CU consumption
+    elif consumed_read_units_percent <= reads_lower_threshold:
+
+        # Exit if down scaling has been disabled
+        if not get_table_option(key_name, 'enable_reads_down_scaling'):
+            logger.debug(
+                '{0} - Down scaling event detected. No action taken as scaling'
+                'down reads has been disabled in the configuration'.format(
+                    table_name))
+        else:
+            if decrease_reads_unit == 'percent':
+                calculated_provisioning = calculators.decrease_reads_in_percent(
+                    updated_read_units,
+                    decrease_reads_with,
+                    get_table_option(key_name, 'min_provisioned_reads'),
+                    table_name)
+            else:
+                calculated_provisioning = calculators.decrease_reads_in_units(
+                    updated_read_units,
+                    decrease_reads_with,
+                    get_table_option(key_name, 'min_provisioned_reads'),
+                    table_name)
+
+            if current_read_units != calculated_provisioning:
+                num_consec_read_checks = num_consec_read_checks + 1
+
+                if num_consec_read_checks >= num_read_checks_before_scale_down:
+                    update_needed = True
+                    updated_read_units = calculated_provisioning
+
+    # Increase needed due to high throttling
     elif throttled_read_count > throttled_reads_upper_threshold:
 
         if throttled_reads_upper_threshold > 0:
@@ -240,28 +287,7 @@ def __ensure_provisioning_reads(table_name, key_name, num_consec_read_checks):
                 update_needed = True
                 updated_read_units = calculated_provisioning
 
-    elif consumed_read_units_percent <= reads_lower_threshold:
-
-        if decrease_reads_unit == 'percent':
-            calculated_provisioning = calculators.decrease_reads_in_percent(
-                updated_read_units,
-                decrease_reads_with,
-                get_table_option(key_name, 'min_provisioned_reads'),
-                table_name)
-        else:
-            calculated_provisioning = calculators.decrease_reads_in_units(
-                updated_read_units,
-                decrease_reads_with,
-                get_table_option(key_name, 'min_provisioned_reads'),
-                table_name)
-
-        if current_read_units != calculated_provisioning:
-            num_consec_read_checks = num_consec_read_checks + 1
-
-            if num_consec_read_checks >= num_read_checks_before_scale_down:
-                update_needed = True
-                updated_read_units = calculated_provisioning
-
+    # Never go over the configured max provisioning
     if max_provisioned_reads:
         if int(updated_read_units) > int(max_provisioned_reads):
             update_needed = True
@@ -298,12 +324,16 @@ def __ensure_provisioning_writes(
 
     update_needed = False
     try:
+        lookback_window_start = get_table_option(
+            key_name, 'lookback_window_start')
         current_write_units = dynamodb.get_provisioned_table_write_units(
             table_name)
         consumed_write_units_percent = \
-            table_stats.get_consumed_write_units_percent(table_name)
+            table_stats.get_consumed_write_units_percent(
+                table_name, lookback_window_start)
         throttled_write_count = \
-            table_stats.get_throttled_write_event_count(table_name)
+            table_stats.get_throttled_write_event_count(
+                table_name, lookback_window_start)
         writes_upper_threshold = \
             get_table_option(key_name, 'writes_upper_threshold')
         writes_lower_threshold = \
@@ -356,30 +386,72 @@ def __ensure_provisioning_writes(
             '{0} - Scaling down writes is not done when usage is at 0%'.format(
                 table_name))
 
+    # Increase needed due to high CU consumption
     elif consumed_write_units_percent >= writes_upper_threshold:
 
-        if increase_writes_unit == 'percent':
-            calculated_provisioning = calculators.increase_writes_in_percent(
-                current_write_units,
-                increase_writes_with,
-                get_table_option(key_name, 'max_provisioned_writes'),
-                table_name)
-        else:
-            calculated_provisioning = calculators.increase_writes_in_units(
-                current_write_units,
-                increase_writes_with,
-                get_table_option(key_name, 'max_provisioned_writes'),
-                table_name)
-
-        if current_write_units != calculated_provisioning:
-            logger.info(
-                '{0} - Resetting the number of consecutive '
-                'write checks. Reason: scale up event detected'.format(
+        # Exit if up scaling has been disabled
+        if not get_table_option(key_name, 'enable_writes_up_scaling'):
+            logger.debug(
+                '{0} - Up scaling event detected. No action taken as scaling '
+                'up writes has been disabled in the configuration'.format(
                     table_name))
-            num_consec_write_checks = 0
-            update_needed = True
-            updated_write_units = calculated_provisioning
+        else:
+            if increase_writes_unit == 'percent':
+                calculated_provisioning = \
+                    calculators.increase_writes_in_percent(
+                        current_write_units,
+                        increase_writes_with,
+                        get_table_option(key_name, 'max_provisioned_writes'),
+                        table_name)
+            else:
+                calculated_provisioning = calculators.increase_writes_in_units(
+                    current_write_units,
+                    increase_writes_with,
+                    get_table_option(key_name, 'max_provisioned_writes'),
+                    table_name)
 
+            if current_write_units != calculated_provisioning:
+                logger.info(
+                    '{0} - Resetting the number of consecutive '
+                    'write checks. Reason: scale up event detected'.format(
+                        table_name))
+                num_consec_write_checks = 0
+                update_needed = True
+                updated_write_units = calculated_provisioning
+
+    # Decrease needed due to low CU consumption
+    elif consumed_write_units_percent <= writes_lower_threshold:
+
+        # Exit if up scaling has been disabled
+        if not get_table_option(key_name, 'enable_writes_down_scaling'):
+            logger.debug(
+                '{0} - Down scaling event detected. No action taken as scaling '
+                'down writes has been disabled in the configuration'.format(
+                    table_name))
+        else:
+            if decrease_writes_unit == 'percent':
+                calculated_provisioning = \
+                    calculators.decrease_writes_in_percent(
+                        current_write_units,
+                        decrease_writes_with,
+                        get_table_option(key_name, 'min_provisioned_writes'),
+                        table_name)
+            else:
+                calculated_provisioning = calculators.decrease_writes_in_units(
+                    current_write_units,
+                    decrease_writes_with,
+                    get_table_option(key_name, 'min_provisioned_writes'),
+                    table_name)
+
+            if current_write_units != calculated_provisioning:
+                num_consec_write_checks = num_consec_write_checks + 1
+
+                if (num_consec_write_checks >=
+                        num_write_checks_before_scale_down):
+                    update_needed = True
+                    updated_write_units = calculated_provisioning
+
+    # Increase needed due to high throttling
     elif throttled_write_count > throttled_writes_upper_threshold:
 
         if throttled_writes_upper_threshold > 0:
@@ -406,28 +478,7 @@ def __ensure_provisioning_writes(
                 update_needed = True
                 updated_write_units = calculated_provisioning
 
-    elif consumed_write_units_percent <= writes_lower_threshold:
-
-        if decrease_writes_unit == 'percent':
-            calculated_provisioning = calculators.decrease_writes_in_percent(
-                current_write_units,
-                decrease_writes_with,
-                get_table_option(key_name, 'min_provisioned_writes'),
-                table_name)
-        else:
-            calculated_provisioning = calculators.decrease_writes_in_units(
-                current_write_units,
-                decrease_writes_with,
-                get_table_option(key_name, 'min_provisioned_writes'),
-                table_name)
-
-        if current_write_units != calculated_provisioning:
-            num_consec_write_checks = num_consec_write_checks + 1
-
-            if num_consec_write_checks >= num_write_checks_before_scale_down:
-                update_needed = True
-                updated_write_units = calculated_provisioning
-
+    # Never go over the configured max provisioning
     if max_provisioned_writes:
         if int(updated_write_units) > int(max_provisioned_writes):
             update_needed = True
@@ -493,3 +544,95 @@ def __update_throughput(table_name, key_name, read_units, write_units):
         key_name,
         int(read_units),
         int(write_units))
+
+
+def __ensure_provisioning_alarm(table_name, key_name):
+    """ Ensure that provisioning alarm threshold is not exceeded
+
+    :type table_name: str
+    :param table_name: Name of the DynamoDB table
+    :type key_name: str
+    :param key_name: Configuration option key name
+    """
+    consumed_read_units_percent = \
+        table_stats.get_consumed_read_units_percent(table_name)
+    consumed_write_units_percent = \
+        table_stats.get_consumed_write_units_percent(table_name)
+
+    reads_upper_alarm_threshold = \
+        get_table_option(key_name, 'reads-upper-alarm-threshold')
+    reads_lower_alarm_threshold = \
+        get_table_option(key_name, 'reads-lower-alarm-threshold')
+    writes_upper_alarm_threshold = \
+        get_table_option(key_name, 'writes-upper-alarm-threshold')
+    writes_lower_alarm_threshold = \
+        get_table_option(key_name, 'writes-lower-alarm-threshold')
+
+    # Check upper alarm thresholds
+    upper_alert_triggered = False
+    upper_alert_message = []
+    if (reads_upper_alarm_threshold > 0 and
+            consumed_read_units_percent >= reads_upper_alarm_threshold):
+        upper_alert_triggered = True
+        upper_alert_message.append(
+            '{0} - Consumed Read Capacity {1:d}% '
+            'was greater than or equal to the upper '
+            'alarm threshold {2:d}%\n'.format(
+                table_name,
+                consumed_read_units_percent,
+                reads_upper_alarm_threshold))
+
+    if (writes_upper_alarm_threshold > 0 and
+            consumed_write_units_percent >= writes_upper_alarm_threshold):
+        upper_alert_triggered = True
+        upper_alert_message.append(
+            '{0} - Consumed Write Capacity {1:d}% '
+            'was greater than or equal to the upper alarm '
+            'threshold {2:d}%\n'.format(
+                table_name,
+                consumed_write_units_percent,
+                writes_upper_alarm_threshold))
+
+    # Check lower alarm thresholds
+    lower_alert_triggered = False
+    lower_alert_message = []
+    if (reads_lower_alarm_threshold > 0 and
+            consumed_read_units_percent < reads_lower_alarm_threshold):
+        lower_alert_triggered = True
+        lower_alert_message.append(
+            '{0} - Consumed Read Capacity {1:d}% '
+            'was below the lower alarm threshold {2:d}%\n'.format(
+                table_name,
+                consumed_read_units_percent,
+                reads_lower_alarm_threshold))
+
+    if (writes_lower_alarm_threshold > 0 and
+            consumed_write_units_percent < writes_lower_alarm_threshold):
+        lower_alert_triggered = True
+        lower_alert_message.append(
+            '{0} - Consumed Write Capacity {1:d}% '
+            'was below the lower alarm threshold {2:d}%\n'.format(
+                table_name,
+                consumed_write_units_percent,
+                writes_lower_alarm_threshold))
+
+    # Send alert if needed
+    if upper_alert_triggered:
+        logger.info(
+            '{0} - Will send high provisioning alert'.format(table_name))
+        sns.publish_table_notification(
+            key_name,
+            ''.join(upper_alert_message),
+            ['high-throughput-alarm'],
+            subject='ALARM: High Throughput for Table {0}'.format(table_name))
+    elif lower_alert_triggered:
+        logger.info(
+            '{0} - Will send low provisioning alert'.format(table_name))
+        sns.publish_table_notification(
+            key_name,
+            ''.join(lower_alert_message),
+            ['low-throughput-alarm'],
+            subject='ALARM: Low Throughput for Table {0}'.format(table_name))
+    else:
+        logger.debug('{0} - Throughput alarm thresholds not crossed'.format(
+            table_name))
